/*
 * The contents of this file are subject to the AOLserver Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://aolserver.lcs.mit.edu/.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is AOLserver Code and related documentation
 * distributed by AOL.
 * 
 * The Initial Developer of the Original Code is America Online,
 * Inc. Portions created by AOL are Copyright (C) 1999 America Online,
 * Inc. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public License (the "GPL"), in which case the
 * provisions of GPL are applicable instead of those above.	If you wish
 * to allow use of your version of this file only under the terms of the
 * GPL and not to allow others to use your version of this file under the
 * License, indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by the GPL.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under either the License or the GPL.
 */

/* 
 * Header file for AS2.1 ADP Module. Most of this is pulled out of AS3b3 nsd.h and ns.h
 */

#ifndef _ADPAS21_H_
#define _ADPAS21_H_


/*
 * Constants
 */
#define ADP_OK       0
#define ADP_BREAK    1
#define ADP_ABORT    2
#define ADP_OVERFLOW 3
#define ADP_RETURN   4

#define NS_CACHE_FREE            ((Ns_Callback *)(-1))

/* 
 * Defines for Unix/NT compatability
 */
#define R_OK 04
#define strncasecmp strnicmp
#define strcasecmp  stricmp


/*
 * "Efficency" macros defined in AS3
 */
#define Ns_StrDup ns_strdup
#define Ns_StrCopy ns_strcopy
#define Ns_Malloc ns_malloc
#define Ns_Realloc ns_realloc
#define Ns_Free ns_free
#define Ns_Calloc ns_calloc
#define STREQ(a,b)              (((*a) == (*b)) && (strcmp((a),(b)) == 0))
#define S_ISREG(m)      (((m)&S_IFMT) == S_IFREG)

/*
 * typdef definitions
 */
typedef void           Ns_Cache;
typedef void           Ns_Entry;
typedef Tcl_HashSearch Ns_CacheSearch;

/* 
 * From AOLserver 3.0b3's ns.h
 */
typedef void  (Ns_AdpParserProc)(Ns_DString *, char *);

/*
 * From nsthread.h
 */
typedef struct Ns_Time {
    time_t      sec;
    long        usec;
} Ns_Time;
typedef void *Ns_Cond;
typedef void *Ns_Pool;

/*
 * The following structure maintains per-thread ADP context.
 */
typedef struct AdpData {
    int	               exception;
    int                depth;
    int                argc;
    char             **argv;
    char              *cwd;
    int                evalLevel;
    int                debugLevel;
    int                debugInit;
    char              *debugFile;
    Ns_DString	       output;
    Ns_Conn           *conn;
    int                fStream;
    Ns_AdpParserProc  *parserProc;
} AdpData;

/*
 * The following structure defines a parsed page in the ADP cache.
 */
typedef struct Page {
    size_t  size;
    time_t  mtime;
    size_t  length;
    void   *pdPtr;	    /* Page data, used by adp81. */
    char    chunks[4];
} Page;


/*
 * ADP routines.
 */
extern char    *NsAdpCopyChunks(Page *pagePtr);
extern int      NsAdpDebugEval(Tcl_Interp *, char *file, char *chunks);
extern int      NsAdpEval(Tcl_Interp *, char *file, char *chunks);
extern void     NsAdpFlush(AdpData *adPtr);
extern void     NsAdpFreeChunks(char *chunks);
extern void     NsAdpFreePage(Page *pagePtr);
extern void    *NsAdpGetCode(Page *pagePtr);
extern AdpData *NsAdpGetData(void);
extern void     NsAdpReleaseCode(Page *pagePtr, void *codePtr);
extern int      NsAdpRun(Tcl_Interp *, char *file, void *codePtr);
extern void     NsAdpParsers(void);
extern int      Ns_AdpRegisterParser(char *extension, Ns_AdpParserProc *newParserProc);


/*
 * ASP-style routines.
 */
extern void     NsAspStyleAdpsInit(char *hServer, char *path);


/*
 * Cache code and routines
 */
extern void NsCacheInit(char *server);
extern Ns_Cache *Ns_CacheCreate(char *name, int keys, time_t timeout,
                                Ns_Callback *freeProc);
extern Ns_Cache *Ns_CacheCreateSz(char *name, int keys, size_t maxSize,
                                  Ns_Callback *freeProc);
extern Ns_Cache *Ns_CacheFind(char *name);
extern void *Ns_CacheMalloc(Ns_Cache *cache, size_t len);
extern void Ns_CacheFree(Ns_Cache *cache, void *bytes);
extern Ns_Entry *Ns_CacheFindEntry(Ns_Cache *cache, char *key);
extern Ns_Entry *Ns_CacheCreateEntry(Ns_Cache *cache, char *key, int *newPtr);
extern char *Ns_CacheName(Ns_Entry *entry);
extern char *Ns_CacheKey(Ns_Entry *entry);
extern void *Ns_CacheGetValue(Ns_Entry *entry);
extern void Ns_CacheSetValue(Ns_Entry *entry, void *value);
extern void Ns_CacheSetValueSz(Ns_Entry *entry, void *value, size_t size);
extern void Ns_CacheUnsetValue(Ns_Entry *entry);
extern void Ns_CacheDeleteEntry(Ns_Entry *entry);
extern void Ns_CacheFlushEntry(Ns_Entry *entry);
extern Ns_Entry *Ns_CacheFirstEntry(Ns_Cache *cache, Ns_CacheSearch *search);
extern Ns_Entry *Ns_CacheNextEntry(Ns_CacheSearch *search);
extern void Ns_CacheFlush(Ns_Cache *cache);
extern void Ns_CacheLock(Ns_Cache *cache);
extern void Ns_CacheUnlock(Ns_Cache *cache);

// Cache functions removed. These function rely on Ns_Cond* functions
// which aren't in 2.1 and would be very difficult to implement. Since
// they're not needed by ADPs we've removed them.
//
//extern void *Ns_CacheTimedGetValue(Ns_Cache *cache, char *key,
//                                   Ns_Time *timePtr, int *okPtr);
//extern int Ns_CacheTimedWait(Ns_Cache *cache, Ns_Time *timePtr);
//extern void Ns_CacheWait(Ns_Cache *cache);
//extern void Ns_CacheSignal(Ns_Cache *cache);
//extern void Ns_CacheBroadcast(Ns_Cache *cache);

extern Ns_Pool *Ns_PoolCreate(char *name);
extern void *Ns_PoolAlloc(Ns_Pool *pool, size_t size);
extern void Ns_PoolFree(Ns_Pool *pool, void *cp);

extern int      NsAdpDebugEval(Tcl_Interp *, char *file, char *chunks);
extern void     NsAdpFancyInit(char *server, char *path);
extern void     NsAdpFreePage(Page *pagePtr);
extern void    *NsAdpGetCode(Page *pagePtr);
extern void     NsAdpReleaseCode(Page *pagePtr, void *codePtr);
extern int      NsAdpRun(Tcl_Interp *, char *file, void *codePtr);



/*
 * New function implemented in str.c in AS3; placed in AdpFancy here;
 * since that is the only file that calls this.
 */
extern char *Ns_StrNStr(char *pattern, char *expression);

/*
 * AOLserver Tcl commands
 */
extern Tcl_CmdProc NsTclAdpEvalCmd;
extern Tcl_CmdProc NsTclIncludeCommand;
extern Tcl_CmdProc NsTclPutsCmd;
extern Tcl_CmdProc NsTclDirCmd;
extern Tcl_CmdProc NsTclBreakCmd;
extern Tcl_CmdProc NsTclTellCmd;
extern Tcl_CmdProc NsTclTruncCmd;
extern Tcl_CmdProc NsTclDumpCmd;
extern Tcl_CmdProc NsTclArgcCmd;
extern Tcl_CmdProc NsTclArgvCmd;
extern Tcl_CmdProc NsTclBindCmd;
extern Tcl_CmdProc NsTclExceptionCmd;
extern Tcl_CmdProc NsTclStreamCmd;

extern Tcl_CmdProc NsTclStatsCmd;
extern Tcl_CmdProc NsTclFlushCmd;
extern Tcl_CmdProc NsTclNamesCmd;
extern Tcl_CmdProc NsTclSizeCmd;

extern Tcl_CmdProc NsTclRegisterTagCmd;
extern Tcl_CmdProc NsTclRegisterAdpCmd;

#endif /* _ADPAS21_H_ */