/*
 * The contents of this file are subject to the AOLserver Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://aolserver.lcs.mit.edu/.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is AOLserver Code and related documentation
 * distributed by AOL.
 * 
 * The Initial Developer of the Original Code is America Online,
 * Inc. Portions created by AOL are Copyright (C) 1999 America Online,
 * Inc. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public License (the "GPL"), in which case the
 * provisions of GPL are applicable instead of those above.	If you wish
 * to allow use of your version of this file only under the terms of the
 * GPL and not to allow others to use your version of this file under the
 * License, indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by the GPL.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under either the License or the GPL.
 */

/* 
 * pool.c --
 *
 *	Routines for implementing pool memory mangement needed by cache.
 *
 *	Based on thread/oldpool.c in AOLserver 3.0 beta 3.	
 *
 */

#include "ns.h"
#include "adpAS21.h"

typedef struct OldPool {
    char name[32];
    int  nalloc;
} OldPool;

Ns_Pool *
Ns_PoolCreate(char *name)
{
    OldPool *poolPtr;
    static Ns_Mutex lock;
    static int npools;

    poolPtr = ns_calloc(sizeof(OldPool), 1);
    if (name != NULL) {
        strncpy(poolPtr->name, name, 31);
        poolPtr->name[31] = '\0';
    } else {
        Ns_LockMutex(&lock);
        sprintf(poolPtr->name, "p%d", npools++);
        Ns_UnlockMutex(&lock);
    }
    return (Ns_Pool *) poolPtr;
}

void *
Ns_PoolAlloc(Ns_Pool *pool, size_t size)
{
    OldPool *poolPtr = (OldPool *) pool;

    if (poolPtr != NULL) {
        ++poolPtr->nalloc;
    }
    return ns_malloc(size);
}


void
Ns_PoolFree(Ns_Pool *pool, void *ptr)
{
    OldPool *poolPtr = (OldPool *) pool;

    if (ptr == NULL) {
        return;
    }
    if (poolPtr != NULL) {
        --poolPtr->nalloc;
    }
    ns_free(ptr);
}